@@ -1,15 +1,16 @@
 # Author: Lukas Vollmerhaus
 # Date: 2020-10-16
 
-import gzip as _gzip
-import numpy as _np
-import signal as _signal
-from multiprocessing import Pool as _Pool
+import gzip
+import numpy as np
+import signal
+from multiprocessing import Pool
 
 # globals
-__THEMIS_IMAGE_SIZE_BYTES = 131072
-__THEMIS_DT = _np.dtype("uint16")
-__THEMIS_DT = __THEMIS_DT.newbyteorder('>')  # force big endian byte ordering
+THEMIS_IMAGE_SIZE_BYTES = 131072
+THEMIS_DT = np.dtype("uint16")
+THEMIS_DT = THEMIS_DT.newbyteorder('>')  # force big endian byte ordering
+THEMIS_EXPECTED_MINUTE_NUM_FRAMES = 20
 
 
 def read(file_list, workers=1):
@@ -24,16 +25,10 @@ def read(file_list, workers=1):
     :return: images, metadata dictionaries, and problematic files
     :rtype: numpy.ndarray, list[dict], list[dict]
     """
-    # pre-allocate array sizes (optimization)
-    predicted_num_frames = len(file_list) * 20
-    images = _np.empty([256, 256, predicted_num_frames], dtype=__THEMIS_DT)
-    metadata_dict_list = [{}] * predicted_num_frames
-    problematic_file_list = []
-
     # set up process pool (ignore SIGINT before spawning pool so child processes inherit SIGINT handler)
-    original_sigint_handler = _signal.signal(_signal.SIGINT, _signal.SIG_IGN)
-    pool = _Pool(processes=workers)
-    _signal.signal(_signal.SIGINT, original_sigint_handler)  # restore SIGINT handler
+    original_sigint_handler = signal.signal(signal.SIGINT, signal.SIG_IGN)
+    pool = Pool(processes=workers)
+    signal.signal(signal.SIGINT, original_sigint_handler)  # restore SIGINT handler
 
     # if input is just a single file name in a string, convert to a list to be fed to the workers
     if isinstance(file_list, str):
@@ -46,11 +41,21 @@ def read(file_list, workers=1):
         data = pool.map(__themis_readfile_worker, file_list)
     except KeyboardInterrupt:
         pool.terminate()  # gracefully kill children
-        return images, metadata_dict_list, problematic_file_list
+        return np.empty((0, 0, 0), dtype=THEMIS_DT), [], []
     else:
         pool.close()
 
-    # reorganize data
+    # derive number of frames to prepare for
+    total_num_frames = 0
+    for i in range(0, len(data)):
+        total_num_frames += data[i][0].shape[2]
+
+    # pre-allocate array sizes
+    images = np.empty([256, 256, total_num_frames], dtype=THEMIS_DT)
+    metadata_dict_list = [{}] * total_num_frames
+    problematic_file_list = []
+
+    # populate data
     list_position = 0
     for i in range(0, len(data)):
         # check if file was problematic
@@ -75,10 +80,10 @@ def read(file_list, workers=1):
 
     # trim unused elements from predicted array sizes
     metadata_dict_list = metadata_dict_list[0:list_position]
-    images = _np.delete(images, range(list_position, predicted_num_frames), axis=2)
+    images = np.delete(images, range(list_position, total_num_frames), axis=2)
 
     # ensure entire array views as uint16
-    images = images.astype(_np.uint16)
+    images = images.astype(np.uint16)
 
     # return
     data = None
@@ -87,7 +92,7 @@ def read(file_list, workers=1):
 
 def __themis_readfile_worker(file):
     # init
-    images = _np.array([])
+    images = np.array([])
     metadata_dict_list = []
     first_frame = True
     metadata_dict = {}
@@ -99,7 +104,7 @@ def __themis_readfile_worker(file):
     # check file extension to see if it's gzipped or not
     try:
         if file.endswith("pgm.gz"):
-            unzipped = _gzip.open(file, mode='rb')
+            unzipped = gzip.open(file, mode='rb')
         elif file.endswith("pgm"):
             unzipped = open(file, mode='rb')
         else:
@@ -120,7 +125,7 @@ def __themis_readfile_worker(file):
             print("Error reading before image data in file '%s'" % (file))
             problematic = True
             metadata_dict_list = []
-            images = _np.array([])
+            images = np.array([])
             error_message = "error reading before image data: %s" % (str(e))
             return images, metadata_dict_list, problematic, file, error_message
 
@@ -174,14 +179,14 @@ def __themis_readfile_worker(file):
             # read image
             try:
                 # read the image size in bytes from the file
-                image_bytes = unzipped.read(__THEMIS_IMAGE_SIZE_BYTES)
+                image_bytes = unzipped.read(THEMIS_IMAGE_SIZE_BYTES)
 
                 # format bytes into numpy array of unsigned shorts (2byte numbers, 0-65536),
                 # effectively an array of pixel values
-                image_np = _np.frombuffer(image_bytes, dtype=__THEMIS_DT)
+                image_np = np.frombuffer(image_bytes, dtype=THEMIS_DT)
 
                 # change 1d numpy array into 256x256 matrix with correctly located pixels
-                image_matrix = _np.reshape(image_np, (256, 256, 1))
+                image_matrix = np.reshape(image_np, (256, 256, 1))
             except Exception as e:
                 print("Failed reading image data frame: %s" % (str(e)))
                 metadata_dict_list.pop()  # remove corresponding metadata entry
@@ -194,7 +199,7 @@ def __themis_readfile_worker(file):
                 images = image_matrix
                 first_frame = False
             else:
-                images = _np.dstack([images, image_matrix])  # depth stack images (on 3rd axis)
+                images = np.dstack([images, image_matrix])  # depth stack images (on 3rd axis)
 
     # close gzip file
     unzipped.close()
